#include "macros.h" // needs to be declared as first entry in .cpp or .h file!
#ifndef __AbstractExercise__
#define __AbstractExercise__
#include <string>
using namespace std;

/**
 * The Abstract Exercise.
 * Superclass of all Exercises
 *
 * &copy; Bernhard Wagner, xmlizer.biz, 2002.
 */
class AbstractExercise {
public:

	/**
	 * Default constructor.
	 */
	AbstractExercise();

	/**
	 * Constructor.
	 *
	 * @param name The name of the Exercise
	 */
	AbstractExercise(string name);

	/**
	 * Destructor.
	 */
	virtual ~AbstractExercise();

	/**
	 * Starts the exercise. (Template Method Design Pattern)
	 */
	void start();

	/**
	 * Gets the exercise's name.
	 *
	 * @return The exercise's name.
	 */
	inline string getName(){ return fName; }

protected:

	/**
	 * Gets the input for this exercise.
	 * To be overridden in subclasses. (Hook Method)
	 */
	virtual void getInput();

	/**
	 * Executes this exercise.
	 * To be overridden in subclasses. (Hook Method)
	 */
	virtual void execute();

private:

	/**
	 * Initializes this Exercise. Used to factor out 
	 * common code of both constructors.
	 */
	void init(string name);
	string fName;
};

#endif
