#include "macros.h" // needs to be declared as first entry in .cpp or .h file!
#ifndef __ExerciseStore__
#define __ExerciseStore__
#include <map>
#include <string>
using namespace std;

class AbstractExercise;

/**
 * The type mapping strings to exercises.
 */
typedef map<string, AbstractExercise*> ExerciseMap_t;

/**
 * The ExerciseStore.
 * Stores Exercises
 *
 * &copy; Bernhard Wagner, xmlizer.biz, 2002.
 */
class ExerciseStore {
public:
	/**
	 * Retrieves an Exercise from the store.
	 *
	 * @param key The key to retrieve the Exercise for
	 * @return The Exercise found for the given key.
	 * 0 if no Exercise was found.
	 */
	AbstractExercise *getExercise(string key);

	/**
	 * Stores an Exercise in the store.
	 *
	 * @param key The key under which to store the Exercise 
	 * @param exercise The Exercise to store
	 * @return always 0.
	 */
	int putExercise(string key, AbstractExercise *exercise);

	/**
	 * Stores an Exercise in the store.
	 *
	 * @param exercise The Exercise to store
	 * @return always 0.
	 */
	int putExercise(AbstractExercise *exercise);

	/**
	 * Retrieves the one and only instance of ExerciseStore. (Singleton Design Pattern)
	 *
	 * @return The one and only instance of ExerciseStore.
	 */
	static ExerciseStore *instance();

	/**
	 * Destroys the one and only instance of ExerciseStore.
	 */
	static void destroyInstance();
private:

	/**
	 * Default constructor. Private: Singleton Design Pattern
	 */
	ExerciseStore(){}

	/**
	 * Releaeses all Exercises in this store.
	 */
	void releaseAllExercises(); 

	/**
	 * The map storing the Exercises.
	 */
	ExerciseMap_t fExerciseMap;

	/**
	 * The one and only instance of ExerciseStore. (Singleton Design Pattern)
	 */
	static ExerciseStore *fInstance;
};

#endif
