#include "macros.h" // needs to be declared as first entry in .cpp or .h file!
#include <string>
#include <iostream>
using namespace std;
#include "AbstractExercise.h"

/**
 * The Exercise.
 *
 * &copy; Bernhard Wagner, xmlizer.biz, 2002.
 */
class GcdExercise : public AbstractExercise {
public:

	/**
	 * Constructor.
	 *
	 * @param name The name of the exercise
	 */
	GcdExercise(string name) : AbstractExercise(name){};

protected:

	/**
	 * Executes this exercise.
	 * To be overridden in subclasses. (Hook Method)
	 */
	void execute();

	/**
	 * Gets the input for this exercise.
	 * To be overridden in subclasses. (Hook Method)
	 */
	void getInput();

private:

	/**
	 * Calculates the gcd of two integers.
	 *
	 * @param a The first number
	 * @param a The second number
	 * @return The GCD
	 */
	static int gcd(int a, int b);

	int fA;
	int fB;

};

/**
 * Instantiates an object of type GcdExercise and stores it in the ExerciseStore.
 * unnamed namespace: file scope (not visible outside this file -> no naming conflicts!)
 * See Bjarne Stroustrup "The C++ Programming Language" Special Edition, Addison-Wesley 2000, p. 200
 */
namespace {
	GcdExercise proto("GcdExercise");
}


void GcdExercise::getInput(){
	cout << "Enter first number :";
	cin >>  fA;
	cout << "Enter second number :";
	cin >>  fB;
}

void GcdExercise::execute(){
	cout << "  GCD(" << fA << ", " << fB << ") = "<< gcd(fA, fB) << endl;
}

int GcdExercise::gcd(int a, int b){
	while(a != b){
		if(a>b){
			a -= b;
		} else {
			b -= a;
		}
	}
	return a;
}
