#include "macros.h" // needs to be declared as first entry in .cpp or .h file!
#include <string>
#include <iostream>
using namespace std;
#include "AbstractExercise.h"

/**
 * The PalindromeExercise.
 *
 * &copy; Bernhard Wagner, xmlizer.biz, 2003.
 */
class PalindromeExercise : public AbstractExercise {
public:

	/**
	 * Constructor.
	 *
	 * @param name The name of the Exercise
	 */
	PalindromeExercise(string name) : AbstractExercise(name){}

protected:

	/**
	 * Gets the input for this exercise.
	 * To be overridden in subclasses. (Hook Method)
	 */
	void execute();

	/**
	 * Gets the input for this exercise.
	 * To be overridden in subclasses. (Hook Method)
	 */
	void getInput();

private:
	string fWord;
	static bool isPalindrome(const string& strToTest);

};
	
/**
 * Instantiates an object of type PalindromeExercise and stores it in the ExerciseStore.
 * unnamed namespace: file scope (not visible outside this file -> no naming conflicts!)
 * See Bjarne Stroustrup "The C++ Programming Language" Special Edition, Addison-Wesley 2000, p. 200
 */
namespace {
	PalindromeExercise proto("PalindromeExercise");
}

void PalindromeExercise::getInput(){
	cout << "Enter string to test: "<< endl;
	cin >> fWord;
}

void PalindromeExercise::execute(){
	cout << "<" << fWord << ">" << " is " << (isPalindrome(fWord) ? "a " : "no ") << "palindrome." << endl;
}

bool PalindromeExercise::isPalindrome(const string& strToTest){
	const int STR_LENGTH = strToTest.length();
	bool stillIsPalindrome = true;

	for(int i=0;i<(STR_LENGTH / 2);i++){
		stillIsPalindrome = strToTest[i] == strToTest[STR_LENGTH-1-i];
		if(!stillIsPalindrome){
			break;
		}
	}
	return stillIsPalindrome;
}