#include "macros.h" // needs to be declared as first entry in .cpp or .h file!
#include <string>
#include <iostream>
using namespace std;
#include "AbstractExercise.h"

/**
 * The Exercise.
 *
 * &copy; Bernhard Wagner, xmlizer.biz, 2002.
 */
class Pounds2KilosExercise : public AbstractExercise {
public:

	/**
	 * Constructor.
	 *
	 * @param name The name of the Pounds2KilosExercise
	 */
	Pounds2KilosExercise(string name) : AbstractExercise(name){};

protected:

	/**
	 * Executes this exercise.
	 * To be overridden in subclasses. (Hook Method)
	 */
	void execute();

	/**
	 * Gets the input for this exercise.
	 * To be overridden in subclasses. (Hook Method)
	 */
	void getInput();

private:

	/**
	 * Calculates kilos from pounds.
	 *
	 * @param pounds The pounds value to be converted to kilos.
	 * @return The kilos
	 */
	static double p2k(double pounds);
	
	/**
	 * Calculates pounds from kilos.
	 *
	 * @param kilos The kilos value to be converted to pounds.
	 * @return The pounds
	 */
	static double k2p(double kilos);

	double fPounds;
	double fKilos;

};
	
/**
 * Instantiates an object of type Pounds2KilosExercise and stores it in the ExerciseStore.
 * unnamed namespace: file scope (not visible outside this file -> no naming conflicts!)
 * See Bjarne Stroustrup "The C++ Programming Language" Special Edition, Addison-Wesley 2000, p. 200
 */
namespace {
	Pounds2KilosExercise proto("Pounds2KilosExercise");
}


void Pounds2KilosExercise::getInput(){
	cout << "Enter pounds :";
	cin >>  fPounds;
	cout << "Enter kilos :";
	cin >>  fKilos;
}

void Pounds2KilosExercise::execute(){
	cout << fPounds <<"  p = "<< p2k(fPounds) << " kg" << endl;
	cout << fKilos  <<" kg = "<< k2p(fKilos)  << " p"  << endl;
}

const double POUND_IN_KILOS = 0.45;

double Pounds2KilosExercise::p2k(double pounds){
	return pounds * POUND_IN_KILOS;
}

double Pounds2KilosExercise::k2p(double kilos){
	return kilos / POUND_IN_KILOS;
}
