#include "macros.h" // needs to be declared as first entry in .cpp or .h file!
#include <iostream>
using namespace std;
#include "Driver.h"

/*! \mainpage Exercise Framework Documentation
 *
 * \section intro Introduction
 *
 * The idea for this framework came up while working with the MS VC++ IDE.
 * The problem was that course attendants didn't want to create a new project
 * for every exercise, but on the other hand each project is only allowed to
 * have one main function.
 * 
 * One approach could be to add functions and change the main method to call
 * the function of the current exercise. Disadvantage of this approach: the
 * main function needs to be changed when a different exercise is to be
 * executed.
 * 
 * This framework in turn allows to add new exercises by simply
 * coding the Exercise as a subclass of AbstractExercise and adding it to the 
 * project. <STRONG>No need to include the
 * file anywhere via <CODE>##include</CODE>.</STRONG>
 *
 * You can choose at runtime which Exercises to run by providing the names of the
 * Exercises to run on the commandline, e.g. (at the command prompt):
 *
 * <CODE>uebung.exe Fahrenheit2CelsuisExercise</CODE>
 *
 * or
 *
 * <CODE>uebung.exe Fahrenheit2CelsuisExercise Exercise2b Exercise3a MyExercise</CODE>
 *
 * \section create Creating Exercises
 *
 * Program your Exercises by subclassing AbstractExercise, overriding the
 * methods <CODE><STRONG>AbstractExercise::getInput()</STRONG></CODE> (for getting Input) and
 * <CODE><STRONG>AbstractExercise::execute()</STRONG></CODE> (for executing the
 * Exercise).
 *
 * The easiest approach is to take the <code>HelloExercise</code> (see below) and replace
 * all occurrences of the string "<code>HelloExercise</code>" by the name of your new Exercise class.
 * Then proceed and replace the implementations of <code>getInput()</code> and <code>execute()</code> by your own.
 *
 * Don't forget to add the following line to the source (<CODE>.cpp</CODE>) of
 * your Exercise:
 *
 * <CODE>namespace {
	MyExercise proto("MyExercise");
}
</CODE>
 *
 * <EM>(replace "MyExercise" by the
 * classname of your Exercise!)</EM>
 *
 * Thus a "Hello World" Example for Exercise would be coded as file <CODE>HelloExample.cpp</CODE> (try it!):
 * <CODE><PRE>
 * ##include "macros.h" // needs to be declared as first entry in .cpp or .h file!
 * ##include <iostream>
 * ##include <string>
 * using namespace std;
 * ##include "AbstractExercise.h"
 * 
 * class HelloExercise : public AbstractExercise {
 * public:
 *     HelloExercise(string name) : AbstractExercise(name){}
 * protected:
 *     void getInput();
 *     void execute();
 * private:
 *     string fUserName;
 * };
 * namespace {
 *	HelloExercise proto("HelloExercise");
 * }
 *
 * void HelloExercise::getInput(){
 *     cout << "your name please :";
 *     cin >> fUserName;
 * }
 *
 * void HelloExercise::execute(){
 *     cout << "Hello "<< fUserName <<", this is "<< getName() << endl;
 * }
 * </PRE></CODE>
 *
 * \section how How
 *
 * Using this framework you can create Exercises just by adding your .cpp file.
 * There's no need to write a .h file and no need to include a file anywhere.
 * Just add your Exercise class adhering to the rules in section Creating&nbsp;Exercises.
 *
 * The basic trick is that each Exercise creates an instance of itself. Thus the
 * constructor of AbstractExercise gets called, which in turn adds the Exercise
 * to the ExerciseStore.
 * 
 * The main function creates a Driver and calls its <CODE><STRONG>Driver::instantiate()</STRONG></CODE>
 * function for each argument given on the command line. The 
 * <CODE><STRONG>Driver::instantiate()</STRONG></CODE> method
 * consults the ExerciseStore to get the Exercise for the given argument. If the
 * Exercise is found, its <CODE><STRONG>AbstractExercise::getInput()</STRONG></CODE> method is called
 * followed by its <CODE><STRONG>AbstractExercise::execute()</STRONG></CODE> method.
 *
 * When the main function has tried all arguments it terminates. The termination of
 * Driver asks the ExerciseStore to delete all its Exercises.
 *
 * <img src="ExerciseFW.gif" />
 *
 *
 * &copy; Bernhard Wagner, xmlizer.biz, 2002-03.
 */


/**
 * The main program
 *
 */
int main(int argc, char** argv)
{
	cout << "Starting..." << endl;
	Driver driver;
	for(int i = 1; i < argc; i++){
		driver.instantiate(argv[i]);
	}
	cout << "...finished." << endl;
	return 0;
}
/*
Wishlist:
- commandline options:
  - help
  - list (lists the currently available exercises)
  - verbose
- choose a logger environment
*/
